<?php

require_once("xml-generator.php");


function spicynodes_navigator_grouptags_create_tables() {
	global $wpdb;
	
	if ( ! empty($wpdb->charset) ) {
		$charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
	}
	if ( ! empty($wpdb->collate) ) {
		$charset_collate .= " COLLATE $wpdb->collate";
	}
	        
	$table_name = "{$wpdb->prefix}spicynodes_navigator_tags_group";
	$wpdb->query("DROP TABLE IF EXISTS $table_name;");
	$wpdb->query("CREATE TABLE $table_name (
		group_id int(4) unsigned NOT NULL auto_increment,
		group_name varchar(200) NOT NULL UNIQUE,
		PRIMARY KEY (group_id)
		) $charset_collate;");

	$table_name = "{$wpdb->prefix}spicynodes_navigator_tags_group_tag";
	$wpdb->query("DROP TABLE IF EXISTS $table_name;");
	$wpdb->query("CREATE TABLE $table_name (
		group_tag_id int(4) unsigned NOT NULL auto_increment,
		group_id int(4) unsigned NOT NULL default 0,
		tag_id bigint(20) unsigned NOT NULL default 0,
		PRIMARY KEY  (group_tag_id)
		) $charset_collate;");
}

function spicynodes_navigator_grouptags_addGroup() {
	global $wpdb;
	
	$table_name = "{$wpdb->prefix}spicynodes_navigator_tags_group";
	$names = $wpdb->get_col( $wpdb->prepare( "SELECT group_name FROM ${table_name}") );
	$NAME_PREFIX = "Group";
	$m = 1;
	if ($names) {
		foreach($names as $name) {
			if (strpos($name, $NAME_PREFIX) !== FALSE) {
				$n = substr($name, strlen($NAME_PREFIX));
				if (is_numeric($n) && ($m<=$n)) {
					$m = $n+1;
				}
			}
		}
	}
	$groupName = $NAME_PREFIX.$m;
	
	$wpdb->query( $wpdb->prepare( "INSERT INTO ${table_name} ( group_name ) VALUES ( %s )", $groupName) );
	$groupId = $wpdb->get_var( $wpdb->prepare( "SELECT group_id FROM ${table_name} WHERE group_name = %s", $groupName ) );
	
	return array($groupId, $groupName);
}

function spicynodes_navigator_grouptags_saveGroup($groupId, $groupName, $tags_str) {
	global $wpdb;
	
	$table_name = "{$wpdb->prefix}spicynodes_navigator_tags_group";
	$wpdb->query( $wpdb->prepare( "UPDATE ${table_name} SET group_name = '%s' WHERE group_id = %d", array($groupName, $groupId) ) ); //reset always
	
	$tags_str = split (",", $tags_str);
	foreach ($tags_str as $tag_str) {
		$tag_str = ltrim($tag_str);
		$tag_str = rtrim($tag_str);
		$tag = get_term_by( "name", $tag_str, "post_tag" );
		$tag_ids[] = $tag->term_id;
	}
	
	$table_name = "{$wpdb->prefix}spicynodes_navigator_tags_group_tag";
	$old_tag_ids = $wpdb->get_col( $wpdb->prepare( "SELECT tag_id FROM ${table_name} WHERE group_id = %d", $groupId ) );
	
	if (!$tag_ids) {
		$tag_ids = array();
	}
	if (!$old_tag_ids) {
		$old_tag_ids = array();
	}
	
	foreach ($old_tag_ids as $old_tag_id) {
		if (!in_array($old_tag_id, $tag_ids)) {
			$delete_tag_ids[] = $old_tag_id;
		}
	}
	foreach ($tag_ids as $tag_id) {
		if (!in_array($tag_id, $old_tag_ids)) {
			$add_tag_ids[] = $tag_id;
		}
	}
	
	if ($add_tag_ids) {
		foreach ($add_tag_ids as $tag_id) {
			$wpdb->query( $wpdb->prepare( "INSERT INTO ${table_name} ( group_id, tag_id ) VALUES ( %d, %d )", array($groupId, $tag_id) ) );
		}
	}
	if ($delete_tag_ids) {
		foreach ($delete_tag_ids as $tag_id) {
			$wpdb->query( $wpdb->prepare( "DELETE FROM ${table_name} WHERE group_id=%d AND tag_id=%d", array($groupId, $tag_id) ) );
		}
	}
	
	spicynodes_navigator_make_tag_xml();
	
	return $groupId;
}

function spicynodes_navigator_grouptags_deleteGroup($groupId) {
	global $wpdb;
	$table_name = "{$wpdb->prefix}spicynodes_navigator_tags_group_tag";
	$wpdb->query( $wpdb->prepare( "DELETE FROM ${table_name} WHERE group_id=%d", $groupId ) );
	$table_name = "{$wpdb->prefix}spicynodes_navigator_tags_group";
	$wpdb->query( $wpdb->prepare( "DELETE FROM ${table_name} WHERE group_id=%d", $groupId ) );
	
	spicynodes_navigator_make_tag_xml();
}

function spicynodes_navigator_grouptags_getGroupBy_groupId($groupId) {
	global $wpdb;
	$table_name = "{$wpdb->prefix}spicynodes_navigator_tags_group";
	$groupName = $wpdb->get_var( $wpdb->prepare( "SELECT group_name FROM ${table_name} WHERE group_id = %d", $groupId) );
	$table_name = "{$wpdb->prefix}spicynodes_navigator_tags_group_tag";
	$tag_ids = $wpdb->get_col( $wpdb->prepare( "SELECT tag_id FROM ${table_name} WHERE group_id = %d", $groupId ) );
	if ($tag_ids) {
		foreach ($tag_ids as $tag_id) {
			$tag = get_tag( $tag_id );
			if ($tag) {
				$tagObjs[] = $tag;
				$tag_names[] = $tag->name;
			}
		}
		$tag_names = join (', ', $tag_names);
	}
	return array("groupId"=>$groupId, "groupName"=>$groupName, "groupTagObjs"=>$tagObjs, "groupTags"=>$tag_names, "groupTagIds"=>$tag_ids);
}

function spicynodes_navigator_grouptags_getGroups() {
	global $wpdb;
	$table_name = "{$wpdb->prefix}spicynodes_navigator_tags_group";
	$group_ids = $wpdb->get_col( $wpdb->prepare( "SELECT group_id FROM ${table_name}") );
	if ($group_ids) {
		foreach ($group_ids as $group_id) {
			$groups[] = spicynodes_navigator_grouptags_getGroupBy_groupId($group_id);
		}
	}
	return $groups;
}

function spicynodes_navigator_grouptags_get_tags($groupId) {
	if ($groupId) {
		$g = spicynodes_navigator_grouptags_getGroupBy_groupId($groupId);
		$grouptags = $g["groupTags"];
	}
	return $grouptags;
}

function spicynodes_navigator_grouptags_header() {
	$siteurl = get_bloginfo('siteurl');
	$header = <<<END
	<div class="wrap nosubsub">
	    <div class="icon32">
	        <img src="$siteurl/wp-content/plugins/spicynodes-plugin-navigator-wp/static-files/grouptags.png" height=36 width=36>
	        </img>
	    </div>
	    <h2>Group tags for SpicyNodes navigator</h2>
	</div>
END;
	return $header;
}

function spicynodes_navigator_grouptags_nav($currentGroupId) {
	$nav = '<h3 id="groups" style="border-bottom:1px solid #cccccc; padding-bottom:0px">';
	$nav .= '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
	
	$groups = spicynodes_navigator_grouptags_getGroups();
	if (!$groups) {
		$nav .= 'No group yet, please add a group!&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
	} else {
		foreach ($groups as $group) {
			$lgroupId = $group["groupId"];
			$lgroupName = $group["groupName"];
			$lgroupTags = $group["groupTags"];
			if ($lgroupId == $currentGroupId) {
				$className = "nav-tab nav-tab-active";
				$href = '';
			} else {
				$className = "nav-tab";
				$href = "href='edit.php?page=sngrouptag&action=switchGroup&groupId=$lgroupId'";
			}
			$nav .= "<a name='group' $href class='$className'>$lgroupName</a>";
		}
	}
	if (count($groups)<7) {
		$nav .= '<input type="button" value="Add a Group" class="button" onclick="javascript:addGroup();"/>';
	}
	$nav .= '</h3>';
	
	return $nav;
}

function spicynodes_navigator_grouptags_tag_cloud() {
	$s = '<div id="term-list-inner">';
	$args = array(
    'smallest'  => 8, 
    'largest'   => 22,
    'unit'      => 'pt', 
    'format'    => 'array',
    'separator' => ' ',
    'number'    => 0xffffff,
    'orderby'   => 'name', 
    'order'     => 'ASC',
    'link'      => 'view', 
    'taxonomy'  => 'post_tag', 
    'echo'      => false );
  $tags = wp_tag_cloud($args);
  if ($tags) {
  	foreach ($tags as $tag) {
	  	$i = strpos($tag, "</a>");
	  	$j = strrpos($tag, ">", $i-strlen($tag))+1;
	  	$t = substr($tag, $j, $i-$j);
	  	$r = "javascript:addtag('$t');";
	  	
	  	$i = strpos($tag, "href=");
	  	$j = strpos($tag, " ", $i);
	  	$t1 = substr($tag, 0, $i);
	  	$t2 = substr($tag, $j);
	  	$s .= "$t1 href=\"$r\" $t2 ";
	  }
	}
  
	$s .= '</div>';
	return $s;
}

function spicynodes_navigator_grouptags_javascript() {
	$js = <<<END
	<script type="text/javascript">
	window.onbeforeunload = askConfirm;
	var isChanged = false;
	function setChanged(v) {
		isChanged = v;
	}	
	function askConfirm(){
		if (isChanged) {
		  return ("You have not saved your group yet.  If you continue, your work will not be saved.");
		}
	}

	function addtag(t) {
		var o = document.getElementById("currentGroupTags");
		var s = t;
		if (o.value) {
			s = o.value+", "+t;
		}
		o.value = s;
		setChanged(true);
	}
	function showChangeGroup() {
		var o = document.getElementById("show_change_group");
		if (o.style.display=="none") {
			o.style.display = "block";
		} else {
			o.style.display = "none";
		}
	}
	function addGroup() {
    var arr = new Array();
    arr = document.getElementsByName("group");
		
		var MAX_GROUPS = 7;
   	if (arr.length >= MAX_GROUPS) {
   		alert("You can not create new group!\\n(This plugin allows up to 7 groups)");
   		return;
   	}
   	
   	window.open("edit.php?page=sngrouptag&action=addGroup", "_self");
   	/*
    var ihtml = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
    for(var i = 0; i < arr.length; i++) {
    	var g = arr[i];
    	ihtml += '<a name="group" href="'+ g.href + '" class="nav-tab">' + g.text + '</a>';
    }
    var n = arr.length+1;
    var newName = 'Group'+n;
    ihtml += '<a name="group" href="javascript:switchGroup('');" class="nav-tab nav-tab-active">'+newName+'</a>';
    ihtml += '<input type="button" value="Add a Group" class="button" onclick="javascript:addGroup();"/>';
    
    var gs = document.getElementById("groups");
    gs.innerHTML = ihtml;
    
    document.getElementById("currentGroupId").value = '';
    document.getElementById("currentGroupName").value = newName;
		
    var tags = document.getElementById("currentGroupTags");
		tags.value = '';
		
		//
		var o = document.getElementById("more-action");
		if (o.style.display == "none") {
			o.style.display = "block";
		}
		o = document.getElementById("change-tags");
		if (o.style.display == "none") {
			o.style.display = "block";
		}
		*/
  }
  function switchGroup(groupId) {
  	window.open("edit.php?page=sngrouptag&currentGroupId="+groupId, "_self");
  	/*
    var arr = new Array();
    arr = document.getElementsByName("group");
		
		//alert saving change of current group
		
    var ihtml = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
    var switchTo = 'javascript:switchGroup('+groupId+');';
    for(var i = 0; i < arr.length; i++) {
    	var g = arr[i];
    	var className = (g.href == switchTo)?'nav-tab nav-tab-active':'nav-tab';
    	ihtml += '<a name="group" href="'+ g.href + '" class="'+className+'">' + g.text + '</a>';
    }
    ihtml += '<input type="button" value="Add a Group" class="button" onclick="javascript:addGroup();"/>';
    
    var gs = document.getElementById("groups");
    gs.innerHTML = ihtml;
    
    var tags = document.getElementById("currentGroupTags");
		tags.value = ''; //change tags accordingly
		*/
  }
  function changeGroupName() {
    var cgn = document.getElementById("changegroupname");
		var newName = cgn.value;
		if (!newName) {
			cgn.focus();
			alert("Please input a name!");
			return;
		}
		
		var arr = new Array();
    arr = document.getElementsByName("group");
		
		var ihtml = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
		var highlitClassName = 'nav-tab nav-tab-active';
    for(var i = 0; i < arr.length; i++) {
    	var g = arr[i];
    	var groupName = (g.className == highlitClassName)?newName:g.text;
    	ihtml += '<a name="group" href="'+ g.href + '" class="'+g.className+'">' + groupName + '</a>';
    }
    ihtml += '<input type="button" value="Add a Group" class="button" onclick="javascript:addGroup();"/>';
    
    var gs = document.getElementById("groups");
    gs.innerHTML = ihtml;
    
    document.getElementById("currentGroupName").value = newName;
    
    showChangeGroup();
    
    setChanged(true);
  }
  function doSaveGroup() {
  	setChanged(false);
  	return true;
  }
  function deleteGroup(groupId) {
  	var yesno = confirm("Are you sure to delete this group?")
		if (yesno){
			window.open("?q=/admin/content/grouptags&page=sngrouptag&action=deleteGroup&groupId="+groupId, "_self");
		}
  }
	</script>
END;
	return $js;
}

function myredirect($url){
    if (!headers_sent()){    //If headers not sent yet... then do php redirect
        header('Location: '.$url); exit;
    }else{                    //If headers are sent... do java redirect... if java disabled, do html redirect.
        echo '<script type="text/javascript">';
        echo 'window.location.href="'.$url.'";';
        echo '</script>';
        echo '<noscript>';
        echo '<meta http-equiv="refresh" content="0;url='.$url.'" />';
        echo '</noscript>'; exit;
    }
}

?>