<?php

define ("SpicyNodes_MaxTopLevelTags", 5);

require_once("related-tags.php");
require_once("xml-generator.php");


function spicynodes_navigator_init() {
	// Check for the required plugin functions. This will prevent fatal
	// errors occurring when you deactivate the dynamic-sidebar plugin.
	if ( !function_exists('register_sidebar_widget') )
		return;
	
	// This is the function that outputs the form to let the users edit
	// the widget's title. It's an optional feature that users cry for.
	function spicynodes_navigator_control() {

		// Get our options and see if we're handling a form submission.
		$options = get_option('spicynodes_navigator');
		if ( !is_array($options) )
			$options = array('title'=>'SpicyNodes Navigator', 
												'archive'=>'checked', 'category'=>'checked', 'tag'=>'checked',
												'width'=>'190', 'height'=>'180',
												'bkgcolor'=>'F8F8FB', 'nodecolor'=>'FFFFFF', 'textcolor'=>'6A6464',
												'language'=>'en', 'relatedtag'=>''); 
		if ( $_POST['navigator-submit'] ) {
			// Remember to sanitize and format use input appropriately.
			$options['title'] = strip_tags(stripslashes($_POST['navigator-title']));
			$options['archive'] = strip_tags(stripslashes($_POST['navigator-archive']));
			$options['category'] = strip_tags(stripslashes($_POST['navigator-category']));
			$options['tag'] = strip_tags(stripslashes($_POST['navigator-tag']));
			$options['width'] = strip_tags(stripslashes($_POST['navigator-width']));
			$options['height'] = strip_tags(stripslashes($_POST['navigator-height']));
			$options['bkgcolor'] = strip_tags(stripslashes($_POST['navigator-bkgcolor']));
			$options['nodecolor'] = strip_tags(stripslashes($_POST['navigator-nodecolor']));
			$options['textcolor'] = strip_tags(stripslashes($_POST['navigator-textcolor']));
			$options['language'] = strip_tags(stripslashes($_POST['navigator-language']));
			$options['relatedtag'] = strip_tags(stripslashes($_POST['related-tag']));
			if (substr($options['bkgcolor'],0,2) == '0x') {
				$options['bkgcolor'] = substr($options['bkgcolor'],2);
			} elseif (substr($options['bkgcolor'],0,1) == '#') {
				$options['bkgcolor'] = substr($options['bkgcolor'],1);
			}
			if (substr($options['nodecolor'],0,2) == '0x') {
				$options['nodecolor'] = substr($options['nodecolor'],2);
			} elseif (substr($options['nodecolor'],0,1) == '#') {
				$options['nodecolor'] = substr($options['nodecolor'],1);
			}
			if (substr($options['textcolor'],0,2) == '0x') {
				$options['textcolor'] = substr($options['textcolor'],2);
			} elseif (substr($options['textcolor'],0,1) == '#') {
				$options['textcolor'] = substr($options['textcolor'],1);
			}
			update_option('spicynodes_navigator', $options);
			
			foreach (array("navigator", "settings", "tags", "categories", "archives") as $f) {
				$f = SpicyNodes_FileRoot."/xml/${f}.xml";
				if (!is_writable($f)) {
					$permission = "<font color='red'><b>Notice:</b> Please update your permissions so settings can be saved.</font><br />";
					$permission .= "<font color='grey'>(chmod ugo+w -R ".SpicyNodes_FileRoot."/xml)</font>";
					break;
				}
			}
			if (!$permission) {
				$dir = SpicyNodes_FileRoot."/xml";
				if (!is_writable($dir)) {
					$permission = "<font color='red'><b>Notice:</b> Please update your permissions so settings can be saved.</font><br />";
					$permission .= "<font color='grey'>(chmod ugo+w -R ".SpicyNodes_FileRoot."/xml)</font>";
				}
			}
			if (!$permission) {
				if ($options['relatedtag'] == 'checked') {
					spicynodes_navigator_remake_all_related_tags_weight();
				}
				spicynodes_navigator_make_settings_xml(); //to update colors
				spicynodes_navigator_remake_xml();
			}
		}

		// Be sure you format your options to be valid HTML attributes.
		$title = htmlspecialchars($options['title'], ENT_QUOTES);
		$archive = htmlspecialchars($options['archive'], ENT_QUOTES);
		$category = htmlspecialchars($options['category'], ENT_QUOTES);
		$tag = htmlspecialchars($options['tag'], ENT_QUOTES);
		$width = htmlspecialchars($options['width'], ENT_QUOTES);
		$height = htmlspecialchars($options['height'], ENT_QUOTES);
		$bkgcolor = htmlspecialchars($options['bkgcolor'], ENT_QUOTES);
		$nodecolor = htmlspecialchars($options['nodecolor'], ENT_QUOTES);
		$textcolor = htmlspecialchars($options['textcolor'], ENT_QUOTES);
		$language = htmlspecialchars($options['language'], ENT_QUOTES);
		$related_tag = htmlspecialchars($options['relatedtag'], ENT_QUOTES);
		
		$ls = array('en'=>'English', 'es'=>'Spanish', 'fr'=>'French',  'pt'=>'Portuguese',
								'de'=>'German',  'pl'=>'Polish',  'ru'=>'Russian', 'zh'=>'China');
		if ( !in_array($language, array_keys($ls)) ) {
			$language = 'en';
		}
		
		// Here is our little form segment. Notice that we don't need a
		// complete form. This will be embedded into the existing form.
		echo '<p style="text-align:left;"><label for="navigator-title"><b>' . __('Title:') . '</b><br />';
		echo '&nbsp;&nbsp;&nbsp;&nbsp;<input style="width: 200px;" id="navigator-title" name="navigator-title" type="text" value="'.$title.'" /></label></p>';
		echo '<p style="text-align:left;"><label for="navigator-title"><b>' . __('Language:') . '</b><br />';
		echo '&nbsp;&nbsp;&nbsp;&nbsp;<select id="navigator-language" name="navigator-language" style="width: 200px;">';
		foreach ($ls as $k=>$v) {
			echo "   <option value='$k' ";
			if ($language == "$k") { echo ' selected'; }
			echo ">$v</option>";
		}
		echo '</select></label><br />';
		echo '&nbsp;&nbsp;&nbsp;&nbsp;</label><font color="grey"><small>(Language for help tooltips)</small></font></p>';
		echo '<p style="text-align:left;"><label for="navigator-dimentions"><b>' . __('Dimensions:') . '</b><br />';
		echo '&nbsp;&nbsp;&nbsp;&nbsp;<input style="width: 60px;" id="navigator-width" name="navigator-width" type="text" value="'.$width.'" />' . __('width') . ' <font color="grey"><small>(e.g., 190)</small></font><br />';
		echo '&nbsp;&nbsp;&nbsp;&nbsp;<input style="width: 60px;" id="navigator-height" name="navigator-height" type="text" value="'.$height.'" />' . __('height') . ' <font color="grey"><small>(e.g., 180)</small></font></label></p>';
    echo '<p style="text-align:left;"><label for="navigator-branches"><b>' . __('Display these branches of nodes:') . '</b><br />';
    if ('checked' == $tag) {
    	echo '&nbsp;&nbsp;&nbsp;&nbsp;<input type="checkbox" name="navigator-tag" value="checked" checked = "'.$tag.'" />' . __('Show Tags') . '<br />';
    } else {
    	echo '&nbsp;&nbsp;&nbsp;&nbsp;<input type="checkbox" name="navigator-tag" value="checked" />' . __('Show Tags') . '<br />';
    }
    if ('checked' == $category) {
    	echo '&nbsp;&nbsp;&nbsp;&nbsp;<input type="checkbox" name="navigator-category" value="checked" checked = "'.$category.'" />' . __('Show Categories') . '<br />';
    } else {
    	echo '&nbsp;&nbsp;&nbsp;&nbsp;<input type="checkbox" name="navigator-category" value="checked" />' . __('Show Categories') . '<br />';
    }
    if ('checked' == $archive) {
    	echo '&nbsp;&nbsp;&nbsp;&nbsp;<input type="checkbox" name="navigator-archive" value="checked" checked = "'.$archive.'" />'. __('Show older posts (Archives)') . '<br />';
    } else {
    	echo '&nbsp;&nbsp;&nbsp;&nbsp;<input type="checkbox" name="navigator-archive" value="checked" />'. __('Show older posts (Archives)') . '<br />';
    }
    echo '&nbsp;&nbsp;&nbsp;&nbsp;</label><font color="grey"><small>(Please select at least one item)</small></font></p>';
    echo '<p style="text-align:left;"><label for="navigator-colors"><b>' . __('Main colors (hex codes):') . '</b><br />';
    echo '&nbsp;&nbsp;&nbsp;&nbsp;<input style="width: 80px;" type="text" id="navigator-bkgcolor" name="navigator-bkgcolor" value="' . $bkgcolor.'" />' . __('background') . '<br />';
    echo '&nbsp;&nbsp;&nbsp;&nbsp;<input style="width: 80px;" type="text" id="navigator-nodecolor" name="navigator-nodecolor" value="' . $nodecolor.'" />' . __('nodes') . '<br />';
    echo '&nbsp;&nbsp;&nbsp;&nbsp;<input style="width: 80px;" type="text" id="navigator-textcolor" name="navigator-textcolor" value="' . $textcolor.'" />' . __('text') . '</label></p>';
    echo '<p style="text-align:left;"><label for="navigator-branches"><b>' . __('Related tags:') . '</b><br />';
    if ('checked' == $related_tag) {
    	echo '&nbsp;&nbsp;&nbsp;&nbsp;<input type="checkbox" name="related-tag" value="checked" checked = "'.$related_tag.'" />' . __('Re-evaluate related tags') . '<br />';
    } else {
    	echo '&nbsp;&nbsp;&nbsp;&nbsp;<input type="checkbox" name="related-tag" value="checked" />' . __('Re-evaluate related tags') . '<br />';
    }
		echo '<input type="hidden" id="navigator-submit" name="navigator-submit" value="1" />';
		if ($permission) {
			echo "<p style='text-align:left;'><small>$permission</small>";	
		}
	}
	
	// This is the function that outputs our little form.
	function spicynodes_navigator_widget($args) {		
		// $args is an array of strings that help widgets to conform to
		// the active theme: before_widget, before_title, after_widget,
		// and after_title are the array keys. Default tags: li and h2.
		extract($args);

		$options = get_option('spicynodes_navigator');
		$title = htmlspecialchars($options['title'], ENT_QUOTES);
		$width = htmlspecialchars($options['width'], ENT_QUOTES);
		$height = htmlspecialchars($options['height'], ENT_QUOTES);
		
		// These lines generate our output. Widgets can be very complex
		// but as you can see here, they can also be very, very simple.
		echo $before_widget . $before_title . $title . $after_title;
		echo '<div>';
		$pageURL = 'http';
		if ($_SERVER["HTTPS"] == "on") {
			$pageURL .= "s";
			}
		$pageURL .= "://";
		if ($_SERVER["SERVER_PORT"] != "80") {
			$pageURL .= $_SERVER["SERVER_NAME"].":".$_SERVER["SERVER_PORT"];
		} else {
			$pageURL .= $_SERVER["SERVER_NAME"];
		}
		$p = SpicyNodes_FileRoot;
		$p = substr($p, strlen($_SERVER['DOCUMENT_ROOT']));
		$p = $pageURL.$p;
		$snconnector = "$p/static-files/snconnector.swf";
		$nodemap = "$p/xml/navigator.xml";
		$settings = "$p/xml/settings.xml";

		$tag = htmlspecialchars($options['tag'], ENT_QUOTES);
		$has_tag = ("checked" == $tag);
		if ($has_tag) {
			/*
			$num = SpicyNodes_MaxTopLevelTags;
			if (is_single()) {
				global $post;
				$tags = get_the_tags($post->ID);
				if ($tags) {
					$tags = array_slice($tags, 0, $num);
				}
			} else {
				$tags = get_tags("orderby=count&order=DESC&number=$num");
			}
			
			if ($tags) {
				foreach ($tags as $tag){
					$tag_ids[] = $tag->term_id;
				}
				$tag_ids = join(",", $tag_ids);
				$nodemap .= $tag_ids;
			}
			*/
			if (is_single()) {
				global $post;
				spicynodes_navigator_make_xml_4_post($post->ID);
				$f = "navigator-post-".$post->ID.".xml";
				$nodemap = "$p/xml/$f";
			} else if (is_tag()) {
				$term_id = get_query_var('tag_id');
				spicynodes_navigator_make_xml_4_tag($term_id);
				$f = "navigator-tag-${term_id}.xml";
				$nodemap = "$p/xml/$f";
			}
		}		

		$s = <<<END
				<object id="spicynodesViewer" classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"
					codebase="http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab" height="$height" width="$width">
					 
					    <param name="movie" value="http://media.spicynodes.org/display.swf"/>
					 
					    <param name="quality" value="high"/>
					 
					    <param name="allowFullScreen" value="true"/>

					    <param name="allowScriptAccess" value="always"/>
					 
					    <param name="flashvars" value="snconnector=$snconnector&nodemap=$nodemap&settings=$settings&wp=true"/>
					 
					    <embed height="$height" width="$width" pluginspage="http://www.macromedia.com/go/getflashplayer" src="http://media.spicynodes.org/display.swf" 
				
								type="application/x-shockwave-flash" quality="high" allowFullScreen="true" allowScriptAccess="always"
				
								flashvars="snconnector=$snconnector&nodemap=$nodemap&settings=$settings&wp=true"/>
				</object>
END;
		echo $s;
		echo '</div>';
		echo $after_widget;
	}

	// This registers our optional widget control form. Because of this
	// our widget will have a button that reveals a 220x175 pixel form.
	register_widget_control(array('SpicyNodes Navigator', 'widgets'), 'spicynodes_navigator_control', 220, 175);
	
	// This registers our widget so it appears with the other available
	// widgets and can be dragged and dropped into any active sidebars.
	register_sidebar_widget(array('SpicyNodes Navigator', 'widgets'), 'spicynodes_navigator_widget');
}

?>